@@ -15,10 +15,12 @@ pub struct DiscreteStateSpace<T: Solver + Default> {
     eigen_frequencies: Option<Vec<(usize, f64)>>,
     max_eigen_frequency: Option<f64>,
     hankel_singular_values_threshold: Option<f64>,
+    #[allow(dead_code)]
     n_io: Option<(usize, usize)>,
     phantom: PhantomData<T>,
     ins: Vec<Box<dyn GetIn>>,
     outs: Vec<Box<dyn GetOut>>,
+    outs_transform: Vec<Option<DMatrix<f64>>>,
 }
 impl<T: Solver + Default> From<FEM> for DiscreteStateSpace<T> {
     /// Creates a state space model builder from a FEM structure
@@ -116,9 +118,36 @@ impl<T: Solver + Default> DiscreteStateSpace<T> {
         Vec<Option<fem_io::Outputs>>: fem_io::FemIo<U>,
         U: 'static + Send + Sync,
     {
-        let mut outs = self.outs;
+        let Self {
+            mut outs,
+            mut outs_transform,
+            ..
+        } = self;
         outs.push(Box::new(SplitFem::<U>::new()));
-        Self { outs, ..self }
+        outs_transform.push(None);
+        Self {
+            outs,
+            outs_transform,
+            ..self
+        }
+    }
+    pub fn outs_with<U>(self, transform: DMatrix<f64>) -> Self
+    where
+        Vec<Option<fem_io::Outputs>>: fem_io::FemIo<U>,
+        U: 'static + Send + Sync,
+    {
+        let Self {
+            mut outs,
+            mut outs_transform,
+            ..
+        } = self;
+        outs.push(Box::new(SplitFem::<U>::new()));
+        outs_transform.push(Some(transform));
+        Self {
+            outs,
+            outs_transform,
+            ..self
+        }
     }
     /// Returns the Hankel singular value for a given eigen mode
     pub fn hankel_singular_value(w: f64, z: f64, b: &[f64], c: &[f64]) -> f64 {
@@ -186,8 +215,13 @@ impl<T: Solver + Default> DiscreteStateSpace<T> {
             let v: Vec<f64> = self
                 .outs
                 .iter_mut()
-                .scan(0usize, |s, x| {
-                    let mat = x.get_out(fem).unwrap();
+                .zip(&self.outs_transform)
+                .scan(0usize, |s, (x, t)| {
+                    let mat = if let Some(t) = t {
+                        t * x.get_out(fem).unwrap()
+                    } else {
+                        x.get_out(fem).unwrap()
+                    };
                     let l = mat.nrows();
                     x.set_range(*s, *s + l);
                     *s += l;
@@ -204,6 +238,7 @@ impl<T: Solver + Default> DiscreteStateSpace<T> {
             None
         }
     }
+    #[allow(dead_code)]
     fn reduce2io(&self, matrix: &DMatrix<f64>) -> Option<DMatrix<f64>> {
         if let Some(fem) = &self.fem {
             let m = DMatrix::from_columns(
@@ -218,7 +253,12 @@ impl<T: Solver + Default> DiscreteStateSpace<T> {
                 &self
                     .outs
                     .iter()
-                    .filter_map(|x| x.trim_out(fem, &m))
+                    .zip(&self.outs_transform)
+                    .filter_map(|(x, t)| match (x.trim_out(fem, &m), t) {
+                        (Some(x), Some(t)) => Some(t * x),
+                        (x, None) => x,
+                        _ => None,
+                    })
                     .flat_map(|x| x.row_iter().map(|x| x.clone_owned()).collect::<Vec<_>>())
                     .collect::<Vec<_>>(),
             ))
@@ -271,7 +311,11 @@ impl<T: Solver + Default> DiscreteStateSpace<T> {
                 log::info!("forces 2 modes: {:?}", forces_2_modes.shape());
                 log::info!("modes 2 nodes: {:?}", modes_2_nodes.shape());
 
+                #[cfg(not(feature = "static-gain"))]
+                let psi_dcg: Option<DMatrix<f64>> = None;
+                #[cfg(feature = "static-gain")]
                 let psi_dcg = if let Some(n_io) = self.n_io {
+                    use std::ops::Range;
                     println!(
                         "The elements of psi_dcg corresponding to 
     - OSSAzDriveTorque
@@ -290,7 +334,9 @@ are set to zero."
                         .static_gain
                         .as_ref()
                         .map(|x| DMatrix::from_row_slice(n_io.1, n_io.0, x));
-                    let static_gain = self.reduce2io(&q.unwrap());
+                    let static_gain = self
+                        .reduce2io(&q.unwrap())
+                        .expect("Failed to produce FEM static gain");
                     let d = na::DMatrix::from_diagonal(&na::DVector::from_row_slice(
                         &w.iter()
                             .skip(3)
@@ -302,18 +348,17 @@ are set to zero."
                     let dyn_static_gain = modes_2_nodes.clone().remove_columns(0, 3)
                         * d
                         * forces_2_modes.clone().remove_rows(0, 3);
-                    let mut psi_dcg = static_gain.unwrap() - dyn_static_gain;
 
-                    let az_torque = self
+                    let mut psi_dcg = static_gain - dyn_static_gain;
+
+                    let az_torque: Option<Range<usize>> = self
                         .ins
                         .iter()
                         .find_map(|x| {
                             x.as_any()
                                 .downcast_ref::<SplitFem<fem_io::OSSAzDriveTorque>>()
                         })
-                        .unwrap()
-                        .range
-                        .clone();
+                        .map(|x| x.range.clone());
                     let az_encoder = self
                         .outs
                         .iter()
@@ -321,9 +366,7 @@ are set to zero."
                             x.as_any()
                                 .downcast_ref::<SplitFem<fem_io::OSSAzEncoderAngle>>()
                         })
-                        .unwrap()
-                        .range
-                        .clone();
+                        .map(|x| x.range.clone());
 
                     let el_torque = self
                         .ins
@@ -332,9 +375,7 @@ are set to zero."
                             x.as_any()
                                 .downcast_ref::<SplitFem<fem_io::OSSElDriveTorque>>()
                         })
-                        .unwrap()
-                        .range
-                        .clone();
+                        .map(|x| x.range.clone());
                     let el_encoder = self
                         .outs
                         .iter()
@@ -342,9 +383,7 @@ are set to zero."
                             x.as_any()
                                 .downcast_ref::<SplitFem<fem_io::OSSElEncoderAngle>>()
                         })
-                        .unwrap()
-                        .range
-                        .clone();
+                        .map(|x| x.range.clone());
 
                     let rot_torque = self
                         .ins
@@ -353,9 +392,7 @@ are set to zero."
                             x.as_any()
                                 .downcast_ref::<SplitFem<fem_io::OSSRotDriveTorque>>()
                         })
-                        .unwrap()
-                        .range
-                        .clone();
+                        .map(|x| x.range.clone());
                     let rot_encoder = self
                         .outs
                         .iter()
@@ -363,12 +400,20 @@ are set to zero."
                             x.as_any()
                                 .downcast_ref::<SplitFem<fem_io::OSSRotEncoderAngle>>()
                         })
-                        .unwrap()
-                        .range
-                        .clone();
+                        .map(|x| x.range.clone());
 
-                    let torque_indices = az_torque.chain(el_torque).chain(rot_torque);
-                    let enc_indices = az_encoder.chain(el_encoder).chain(rot_encoder);
+                    let torque_indices: Vec<_> = az_torque
+                        .into_iter()
+                        .chain(el_torque.into_iter())
+                        .chain(rot_torque.into_iter())
+                        .flat_map(|x| x.collect::<Vec<usize>>())
+                        .collect();
+                    let enc_indices: Vec<_> = az_encoder
+                        .into_iter()
+                        .chain(el_encoder.into_iter())
+                        .chain(rot_encoder.into_iter())
+                        .flat_map(|x| x.collect::<Vec<usize>>())
+                        .collect();
 
                     for i in torque_indices {
                         for j in enc_indices.clone() {
